// forin.cs
//
// Copyright 2010 Microsoft Corporation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

using System.Collections.Generic;

namespace Microsoft.Ajax.Utilities
{
    public sealed class ForIn : IterationStatement
    {
        private AstNode m_variable;
        private AstNode m_collection;

        public AstNode Variable
        {
            get { return m_variable; }
            set
            {
                m_variable.IfNotNull(n => n.Parent = (n.Parent == this) ? null : n.Parent);
                m_variable = value;
                m_variable.IfNotNull(n => n.Parent = this);
            }
        }

        public AstNode Collection
        {
            get { return m_collection; }
            set
            {
                m_collection.IfNotNull(n => n.Parent = (n.Parent == this) ? null : n.Parent);
                m_collection = value;
                m_collection.IfNotNull(n => n.Parent = this);
            }
        }

        public Context OperatorContext { get; set; }

        public BlockScope BlockScope { get; set; }

        public override Context TerminatingContext
        {
            get
            {
                // if we have one, return it. If not, return what the body has (if any)
                return base.TerminatingContext ?? Body.IfNotNull(b => b.TerminatingContext);
            }
        }

        public ForIn(Context context)
            : base(context)
        {
        }

        public override void Accept(IVisitor visitor)
        {
            if (visitor != null)
            {
                visitor.Visit(this);
            }
        }

        public override IEnumerable<AstNode> Children
        {
            get
            {
                return EnumerateNonNullNodes(Variable, Collection, Body);
            }
        }

        public override bool ReplaceChild(AstNode oldNode, AstNode newNode)
        {
            if (Variable == oldNode)
            {
                Variable = newNode;
                return true;
            }
            if (Collection == oldNode)
            {
                Collection = newNode;
                return true;
            }
            if (Body == oldNode)
            {
                Body = ForceToBlock(newNode);
                return true;
            }
            return false;
        }

        internal override bool EncloseBlock(EncloseBlockType type)
        {
            // pass the query on to the body
            return Body == null ? false : Body.EncloseBlock(type);
        }
    }
}
